from dataclasses import dataclass
from uuid import UUID

from praetorian_api_client.resources.base import BaseResource


class TokenResource(BaseResource):
    @dataclass
    class Token(object):
        token: UUID
        active_2fa: bool

    def auth(self, username: str, password: str) -> Token:
        payload = self.fill_content(username=username, password=password)
        response = self.requestor.request('POST', 'tokens/', payload=payload)['response']

        return self.Token(
            token=response.get('token'),
            active_2fa=response.get('active_2fa')
        )



from dataclasses import dataclass
from typing import List
from uuid import UUID

from praetorian_api_client.resources.base import BaseResource


class ServiceResource(BaseResource):
    @dataclass
    class Service(object):
        id: UUID
        name: str
        type: str
        variables: dict

    def list(self, remote_id: str = None, service_type: str = None) -> List[Service]:
        query = self.fill_content(remote_id=remote_id, service_type=service_type)
        response = self.requestor.request('GET', 'services/', query=query)['items']

        return [
            self.Service(
                id=item.get('id'),
                name=item.get('name'),
                type=item.get('type'),
                variables=item.get('variables')
            ) for item in response
        ]



from http import HTTPStatus


class ApiException(Exception):
    def __init__(
        self,
        request,
        message: str,
        status_code: int = HTTPStatus.INTERNAL_SERVER_ERROR,
        previous: Exception = None
    ):
        super().__init__(message)

        self._request = request
        self._status_code = status_code
        self._message = message
        self._previous = previous

    @property
    def request(self):
        return self._request

    @property
    def status_code(self) -> int:
        return self._status_code

    @property
    def message(self) -> str:
        return self._message

    @property
    def previous(self) -> Exception:
        return self._previous

    @property
    def payload(self) -> dict:
        result = {
            'message': self.message,
            'code': self.status_code
        }

        return result


class InvalidSignatureGenerated(ApiException):
    def __init__(self, secret: str, message: str, signature: str):
        self._secret = secret
        self._message = message
        self._signature = signature

    @property
    def secret(self) -> str:
        return self._secret

    @property
    def message(self) -> str:
        return self._message

    @property
    def signature(self) -> str:
        return self._signature



import json
import uuid

from datetime import datetime, date
from decimal import Decimal


class JSONEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, datetime):
            return obj.date().isoformat()
        elif isinstance(obj, date):
            return obj.isoformat()
        elif isinstance(obj, Decimal):
            return str(obj)
        elif isinstance(obj, uuid.UUID):
            return str(obj)
        else:
            return super(JSONEncoder, self).default(obj)



import json
from typing import Union, Dict

from requests import Request, Session, RequestException, HTTPError

from .configuration import Configuration
from .errors import ApiException
from .utils import JSONEncoder
from . import version


class Requestor(object):
    def __init__(self, configuration: Configuration):
        self._user_agent = f"praetorian_api_client/{version.__version__}"
        self._configuration = configuration
        self._session = Session()

    def request(self, method: str, endpoint_url: str, payload: dict = None, query: dict = None, parse: bool = True) -> Union[str, Dict]:
        req = Request(
            method=method,
            url=self._configuration.environment.api_url + endpoint_url,
            headers={
                "User-Agent": self._user_agent,
                "X-ApiKey": self._configuration.key,
                "Authorization": f"TOKEN {self._configuration.token}",
            },
        )

        if payload and isinstance(payload, dict):
            req.data = json.dumps(payload, cls=JSONEncoder)
            req.headers['Content-Type'] = 'application/json'

        if query and isinstance(query, dict):
            req.params = query

        prepared = req.prepare()

        try:
            r = self._session.send(
                prepared,
                timeout=self._configuration.timeout
            )
        except RequestException:
            raise ApiException(prepared, 'Request to server timed out.')

        try:
            r.raise_for_status()
        except HTTPError as e:
            raise ApiException(prepared, e.args[0], previous=e)

        data = r.text

        if parse:
            try:
                data = json.loads(data)
            except json.JSONDecodeError:
                raise ApiException(prepared, 'Cannot parse response data to json.')

        return data

    def __str__(self) -> str:
        return self._user_agent



from dataclasses import dataclass
from typing import List
from uuid import UUID

from praetorian_api_client.resources.base import BaseResource


class ProjectResource(BaseResource):
    @dataclass
    class Project(object):
        id: UUID
        name: str
        is_vpn: bool

    def list(self, user_id: str = None) -> List[Project]:
        query = self.fill_content(user_id=user_id)
        response = self.requestor.request('GET', 'projects/', query=query)['items']

        return [
            self.Project(
                id=item.get('id'),
                name=item.get('name'),
                is_vpn=item.get('is_vpn')
            ) for item in response
        ]



from dataclasses import dataclass
from typing import List
from uuid import UUID

from praetorian_api_client.resources.base import BaseResource


class RemoteResource(BaseResource):
    @dataclass
    class Remote(object):
        id: UUID
        name: str
        host: str
        port: str

    def list(self, project_id: str = None, name: str = None) -> List[Remote]:
        query = self.fill_content(project_id=project_id, name=name)
        response = self.requestor.request('GET', 'remotes/', query=query)['items']

        return [
            self.Remote(
                id=item.get('id'),
                name=item.get('name'),
                host=item.get('host'),
                port=item.get('port')
            ) for item in response
        ]



import os

from dotenv import load_dotenv


class Environment(object):
    def __init__(self, name: str, api_url: str, read_only: bool = True):
        self._name = name
        self._api_url = api_url
        self._read_only = read_only

    @property
    def name(self) -> str:
        return self._name

    @property
    def api_url(self) -> str:
        return self._api_url

    @property
    def read_only(self) -> bool:
        return self._read_only

    def __str__(self):
        return self._name

    def __repr__(self):
        return f"{self._name} ({self._api_url})"


class Configuration(object):
    def __init__(
            self, environment: Environment,
            key: str,
            secret: str,
            timeout: int = 30,
            token: str = None
    ):
        self._environment = environment
        self._key = key
        self._secret = secret
        self._timeout = timeout
        self._token = token

    def set_token(self, token: str):
        self._token = token

    @property
    def environment(self) -> Environment:
        return self._environment

    @property
    def key(self) -> str:
        return self._key

    @property
    def secret(self) -> str:
        return self._secret

    @property
    def timeout(self) -> int:
        return self._timeout

    @property
    def token(self) -> str:
        return self._token



from dataclasses import dataclass
from uuid import UUID

from praetorian_api_client.resources.base import BaseResource


class UserResource(BaseResource):
    @dataclass
    class User(object):
        id: UUID
        username: str
        name: str
        surname: str
        email: str
        phone: str
        role: str
        is_temporary: bool
        additional_data: dict

    def get_me(self) -> User:
        response = self.requestor.request('GET', 'users/me/')['response']

        return self.User(
            id=response.get('id'),
            username=response.get('username'),
            name=response.get('name'),
            surname=response.get('surname'),
            email=response.get('email'),
            phone=response.get('phone'),
            role=response.get('role'),
            is_temporary=response.get('is_temporary'),
            additional_data=response.get('additional_data')
        )

    def delete_me(self) -> bool:
        response = self.requestor.request('DELETE', f'users/me/', parse=False)
        is_deleted = False

        if response is not None:
            is_deleted = True

        return is_deleted



from setuptools import setup


def read_files(files):
    data = []
    for file in files:
        with open(file, encoding='utf-8') as f:
            data.append(f.read())
    return "\n".join(data)


meta = {}
with open('praetorian_api_client/version.py') as f:
    exec(f.read(), meta)

setup(
    name='praetorian-api-client',
    version=meta['__version__'],
    packages=[
        'praetorian_api_client',
        'praetorian_api_client.resources'
    ],
    install_requires=[
        'python-dotenv==0.15.*',
        'requests==2.24.*'
    ],
    url='https://github.com/Praetorian-Defence/praetorian-api-client',
    license='MIT',
    author='Adam Žúrek',
    author_email='adamzurek14@gmail.com',
    description='Api client maintaining communication to praetorian-api.',
    long_description=read_files(['README.md', 'CHANGELOG.md']),
    long_description_content_type='text/markdown',
    classifiers=[
        # As from https://pypi.python.org/pypi?%3Aaction=list_classifiers
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Topic :: Database',
        'Topic :: Security'
    ]
)



from .resources.project import ProjectResource
from .resources.remote import RemoteResource
from .resources.service import ServiceResource
from .resources.token import TokenResource
from .resources.user import UserResource
from .version import __version__

from .configuration import Configuration
from .requestor import Requestor


class ApiClient(object):
    def __init__(self, configuration: Configuration):
        self._configuration = configuration
        self._requestor = Requestor(configuration)

        self._user = UserResource(self._requestor, self._configuration)
        self._project = ProjectResource(self._requestor, self._configuration)
        self._remote = RemoteResource(self._requestor, self._configuration)
        self._service = ServiceResource(self._requestor, self._configuration)

    @classmethod
    def create_from_auth(cls, configuration: Configuration, username: str, password: str) -> 'ApiClient':
        token_resource = TokenResource(Requestor(configuration), configuration)
        token_obj = token_resource.auth(username, password)
        configuration.set_token(str(token_obj.token))

        return ApiClient(configuration)

    @property
    def version(self) -> str:
        return __version__

    @property
    def requestor(self) -> Requestor:
        return self._requestor

    @property
    def configuration(self) -> Configuration:
        return self._configuration

    @property
    def user(self) -> UserResource:
        return self._user

    @property
    def project(self) -> ProjectResource:
        return self._project

    @property
    def remote(self) -> RemoteResource:
        return self._remote

    @property
    def service(self) -> ServiceResource:
        return self._service

    def __str__(self) -> str:
        return f"{self.requestor}"



import hashlib
import hmac
from abc import ABC

from praetorian_api_client.configuration import Configuration
from praetorian_api_client.requestor import Requestor


class BaseResource(ABC):
    def __init__(self, requestor: Requestor, configuration: Configuration):
        self._requestor = requestor
        self._configuration = configuration

    def _sign(self, data: str) -> str:
        return hmac.new(self._configuration.secret.encode(), data.encode(), hashlib.sha256).hexdigest().upper()

    @staticmethod
    def fill_content(**kwargs) -> dict:
        values = {}
        for k in kwargs:
            if kwargs[k] is not None:
                values[k] = kwargs[k]
        return values

    @property
    def requestor(self) -> Requestor:
        return self._requestor

    @property
    def configuration(self) -> Configuration:
        return self._configuration
